return {
  {
    'neovim/nvim-lspconfig',

    dependencies = { 'williamboman/mason-lspconfig.nvim' },

    config = function()
      local lconf = require('lspconfig')
      lconf.util.default_config = vim.tbl_extend(
        'force',
        lconf.util.default_config,
        { capabilities = require('cmp_nvim_lsp').default_capabilities() }
      )
    end,

    keys = {
      -- overrides for default keybinds, enhancing with LSP magic
      { 'K',      vim.lsp.buf.hover,           mode = 'n', desc = 'Hover documentation on the object under the cursor' },
      { '<c-]>',  vim.lsp.buf.definition,      mode = 'n', desc = 'Jump to the definition of the object under the cursor' },
      { '<c-[>',  vim.lsp.buf.references,      mode = 'n', desc = 'Open references in the Quickfix list' },
      { '<c-\\>', vim.lsp.buf.type_definition, mode = 'n', desc = 'Jump to the type definition of the object under the cursor' },

      -- auto-format
      {
        '<c-f>',
        function()
          local params = vim.lsp.util.make_range_params()
          params.context = { only = { "source.organizeImports" } }

          local output = vim.lsp.buf_request_sync(0, "textDocument/codeAction", params)
          for clientId, result in pairs(output or {}) do
            for _, r in pairs(result.result or {}) do
              if r.edit then
                vim.lsp.util.apply_workspace_edit(
                  r.edit,
                  (vim.lsp.get_client_by_id(clientId) or {}).offset_encoding or "utf-16"
                )
              end
            end
          end

          vim.lsp.buf.format()
        end,
        mode = 'n',
        desc = 'Run applicable formatters on the current buffer',
      },

      { '<leader>L', '<cmd>LspInfo<cr>', mode = 'n', desc = 'Show current LSP status' },
    },
  },
  {
    'williamboman/mason-lspconfig.nvim',

    dependencies = { 'williamboman/mason.nvim' },

    config = function(_, opts)
      local c = require('mason-lspconfig')
      c.setup_handlers({
        function(server_name)
          require('lspconfig')[server_name].setup {}
        end,
        ['lua_ls'] = function()
          require('lspconfig').lua_ls.setup({
            settings = {
              Lua = {
                diagnostics = {
                  globals = { "vim" }
                }
              }
            }
          })
        end,
        ['gopls'] = function()
          require('lspconfig').gopls.setup({
            settings = {
              gopls = {
                analyses = { unusedparams = true },
                staticcheck = true,
                gofumpt = true,
              },
            },
          })
        end,
        ['ltex'] = function()
          require('lspconfig').ltex.setup {
            cmd = { "/usr/local/bin/ltex-ls" },
            filetypes = {
              'bib', 'gitcommit', 'markdown', 'org',
              'plaintex', 'rst', 'rnoweb', 'tex', 'pandoc',
              'quarto', 'rmd', 'gemtext', 'text',
            },
            flags = { debounce_text_changes = 300 },
          }
        end,
      })
      c.setup(opts)
    end,
  },
  {
    'williamboman/mason.nvim',

    config = function(_, opts)
      require('mason').setup(opts)
    end,

    keys = {
      { '<leader>M', '<cmd>Mason<cr>', mode = 'n', desc = 'Open Mason UI' },
    },
  },
}
