local o = vim.opt
local l = vim.opt_local

-- show relative line numbers
o.number = true
o.relativenumber = true

-- concealing
o.conceallevel = 2
o.concealcursor = 'nc'

-- more color support
o.termguicolors = true

-- min number width to 2
o.numberwidth = 2

-- support mouse everywhere
o.mouse = 'a'

-- round > and < commands to nearest tabstop
o.shiftround = true

-- shorter update time
o.updatetime = 200

-- don't show |ins-completion-menu| messages
o.shortmess = o.shortmess + 'c'

-- always show the signcolumn
o.signcolumn = 'yes'

-- additional patterns to skip in glob matching
o.wildignore = o.wildignore + { '*.pyc', '*.sw?', '*/.mypy_cache/*' }

-- customize the completion menu
o.completeopt = { 'menu', 'menuone', 'noselect' }

-- timeouts
o.timeout = true
o.timeoutlen = 250

-- use a global dictionary file
if (vim.fn.filereadable('/usr/share/dict/words'))
then
  o.dictionary = '/usr/share/dict/words'
end

-- preserve cursor position
vim.api.nvim_create_autocmd('BufReadPost', {
  pattern = { '*' },
  callback = function()
    local line = vim.fn.line
    if (line("'\"") >= 1 and line("'\"") <= line("$"))
    then
      vim.api.nvim_exec("normal! g`\"", false)
    end
  end,
})

-- user commands to set hard tabs or spaces and tabstop width
local function settabs(expand)
  return function(conf)
    local n = tonumber(conf.args)
    l.expandtab = expand
    l.tabstop = n
    l.shiftwidth = n
    l.softtabstop = -1
  end
end
vim.api.nvim_create_user_command("Tabs", settabs(false), { nargs = 1 })
vim.api.nvim_create_user_command("Spaces", settabs(true), { nargs = 1 })

_G.tabs = function(width) settabs(false)({ args = width }) end
_G.spaces = function(width) settabs(true)({ args = width }) end


-- set grep program to ripgrep
if vim.fn.executable('rg')
then
  o.grepprg = 'rg --vimgrep'
  o.grepformat = '%f:%l:%c:%m,%f:%l:%m'
end
